==> ruli-0.17.tar.gz: ruli-0.17/src/ruli_isaac.c <==
/*

ISAAC: a fast cryptographic random number generator
http://burtleburtle.net/bob/rand/isaacafa.html

------------------------------------------------------------------------------
rand.c: By Bob Jenkins.  My random number generator, ISAAC.  Public Domain
MODIFIED:
  960327: Creation (addition of randinit, really)
  970719: use context, not global variables, for internal state
  980324: make a portable version
  010626: Note this is public domain
------------------------------------------------------------------------------

$Id: ruli_isaac.c,v 1.1 2003/05/07 15:40:18 evertonm Exp $

*/


#include "ruli_isaac.h"


#define isaac_ind(mm,x)  ((mm)[(x>>2)&(ISAAC_RANDSIZ-1)])
#define isaac_rngstep(mix,a,b,mm,m,m2,r,x) \
{ \
  x = *m;  \
  a = ((a^(mix)) + *(m2++)) & 0xffffffff; \
  *(m++) = y = (isaac_ind(mm,x) + a + b) & 0xffffffff; \
  *(r++) = b = (isaac_ind(mm,y>>ISAAC_RANDSIZL) + x) & 0xffffffff; \
}

void     isaac(ctx)
isaac_randctx *ctx;
{
   register isaac_ub4 a,b,x,y,*m,*mm,*m2,*r,*mend;
   mm=ctx->randmem; r=ctx->randrsl;
   a = ctx->randa; b = (ctx->randb + (++ctx->randc)) & 0xffffffff;
   for (m = mm, mend = m2 = m+(ISAAC_RANDSIZ/2); m<mend; )
   {
      isaac_rngstep( a<<13, a, b, mm, m, m2, r, x);
      isaac_rngstep( a>>6 , a, b, mm, m, m2, r, x);
      isaac_rngstep( a<<2 , a, b, mm, m, m2, r, x);
      isaac_rngstep( a>>16, a, b, mm, m, m2, r, x);
   }
   for (m2 = mm; m2<mend; )
   {
      isaac_rngstep( a<<13, a, b, mm, m, m2, r, x);
      isaac_rngstep( a>>6 , a, b, mm, m, m2, r, x);
      isaac_rngstep( a<<2 , a, b, mm, m, m2, r, x);
      isaac_rngstep( a>>16, a, b, mm, m, m2, r, x);
   }
   ctx->randb = b; ctx->randa = a;
}


#define isaac_mix(a,b,c,d,e,f,g,h) \
{ \
   a^=b<<11; d+=a; b+=c; \
   b^=c>>2;  e+=b; c+=d; \
   c^=d<<8;  f+=c; d+=e; \
   d^=e>>16; g+=d; e+=f; \
   e^=f<<10; h+=e; f+=g; \
   f^=g>>4;  a+=f; g+=h; \
   g^=h<<8;  b+=g; h+=a; \
   h^=a>>9;  c+=h; a+=b; \
}

/* if (flag==TRUE), then use the contents of randrsl[] to initialize mm[]. */
void isaac_randinit(ctx, flag)
isaac_randctx *ctx;
isaac_word     flag;
{
   isaac_word i;
   isaac_ub4 a,b,c,d,e,f,g,h;
   isaac_ub4 *m,*r;
   ctx->randa = ctx->randb = ctx->randc = 0;
   m=ctx->randmem;
   r=ctx->randrsl;
   a=b=c=d=e=f=g=h=0x9e3779b9;  /* the golden ratio */

   for (i=0; i<4; ++i)          /* scramble it */
   {
     isaac_mix(a,b,c,d,e,f,g,h);
   }

   if (flag) 
   {
     /* initialize using the contents of r[] as the seed */
     for (i=0; i<ISAAC_RANDSIZ; i+=8)
     {
       a+=r[i  ]; b+=r[i+1]; c+=r[i+2]; d+=r[i+3];
       e+=r[i+4]; f+=r[i+5]; g+=r[i+6]; h+=r[i+7];
       isaac_mix(a,b,c,d,e,f,g,h);
       m[i  ]=a; m[i+1]=b; m[i+2]=c; m[i+3]=d;
       m[i+4]=e; m[i+5]=f; m[i+6]=g; m[i+7]=h;
     }
     /* do a second pass to make all of the seed affect all of m */
     for (i=0; i<ISAAC_RANDSIZ; i+=8)
     {
       a+=m[i  ]; b+=m[i+1]; c+=m[i+2]; d+=m[i+3];
       e+=m[i+4]; f+=m[i+5]; g+=m[i+6]; h+=m[i+7];
       isaac_mix(a,b,c,d,e,f,g,h);
       m[i  ]=a; m[i+1]=b; m[i+2]=c; m[i+3]=d;
       m[i+4]=e; m[i+5]=f; m[i+6]=g; m[i+7]=h;
     }
   }
   else
   {
     for (i=0; i<ISAAC_RANDSIZ; i+=8)
     {
       /* fill in mm[] with messy stuff */
       isaac_mix(a,b,c,d,e,f,g,h);
       m[i  ]=a; m[i+1]=b; m[i+2]=c; m[i+3]=d;
       m[i+4]=e; m[i+5]=f; m[i+6]=g; m[i+7]=h;
     }
   }

   isaac(ctx);            /* fill in the first set of results */
   ctx->randcnt=ISAAC_RANDSIZ;  /* prepare to use the first set of results */
}


#ifdef NEVER
int main()
{
  isaac_ub4 i,j;
  isaac_randctx ctx;
  ctx.randa=ctx.randb=ctx.randc=(isaac_ub4)0;
  for (i=0; i<256; ++i) ctx.randrsl[i]=(isaac_ub4)0;
  isaac_randinit(&ctx, ISAAC_TRUE);
  for (i=0; i<2; ++i)
  {
    isaac(&ctx);
    for (j=0; j<256; ++j)
    {
      printf("%.8lx",ctx.randrsl[j]);
      if ((j&7)==7) printf("\n");
    }
  }
}
#endif

==> ruli_0.36.orig.tar.gz: ruli-0.36/src/ruli_isaac.c <==
/*

ISAAC: a fast cryptographic random number generator
http://burtleburtle.net/bob/rand/isaacafa.html

------------------------------------------------------------------------------
rand.c: By Bob Jenkins.  My random number generator, ISAAC.  Public Domain
MODIFIED:
  960327: Creation (addition of randinit, really)
  970719: use context, not global variables, for internal state
  980324: make a portable version
  010626: Note this is public domain
------------------------------------------------------------------------------

$Id: ruli_isaac.c,v 1.2 2004/05/21 18:23:50 evertonm Exp $

*/


#include "ruli_isaac.h"


#define isaac_ind(mm,x)  ((mm)[(x>>2)&(ISAAC_RANDSIZ-1)])
#define isaac_rngstep(mix,a,b,mm,m,m2,r,x) \
{ \
  x = *m;  \
  a = ((a^(mix)) + *(m2++)) & 0xffffffff; \
  *(m++) = y = (isaac_ind(mm,x) + a + b) & 0xffffffff; \
  *(r++) = b = (isaac_ind(mm,y>>ISAAC_RANDSIZL) + x) & 0xffffffff; \
}

void isaac(isaac_randctx *ctx)
{
   register isaac_ub4 a,b,x,y,*m,*mm,*m2,*r,*mend;
   mm=ctx->randmem; r=ctx->randrsl;
   a = ctx->randa; b = (ctx->randb + (++ctx->randc)) & 0xffffffff;
   for (m = mm, mend = m2 = m+(ISAAC_RANDSIZ/2); m<mend; )
   {
      isaac_rngstep( a<<13, a, b, mm, m, m2, r, x);
      isaac_rngstep( a>>6 , a, b, mm, m, m2, r, x);
      isaac_rngstep( a<<2 , a, b, mm, m, m2, r, x);
      isaac_rngstep( a>>16, a, b, mm, m, m2, r, x);
   }
   for (m2 = mm; m2<mend; )
   {
      isaac_rngstep( a<<13, a, b, mm, m, m2, r, x);
      isaac_rngstep( a>>6 , a, b, mm, m, m2, r, x);
      isaac_rngstep( a<<2 , a, b, mm, m, m2, r, x);
      isaac_rngstep( a>>16, a, b, mm, m, m2, r, x);
   }
   ctx->randb = b; ctx->randa = a;
}


#define isaac_mix(a,b,c,d,e,f,g,h) \
{ \
   a^=b<<11; d+=a; b+=c; \
   b^=c>>2;  e+=b; c+=d; \
   c^=d<<8;  f+=c; d+=e; \
   d^=e>>16; g+=d; e+=f; \
   e^=f<<10; h+=e; f+=g; \
   f^=g>>4;  a+=f; g+=h; \
   g^=h<<8;  b+=g; h+=a; \
   h^=a>>9;  c+=h; a+=b; \
}

/* if (flag==TRUE), then use the contents of randrsl[] to initialize mm[]. */
void isaac_randinit(isaac_randctx *ctx, isaac_word flag)
{
   isaac_word i;
   isaac_ub4 a,b,c,d,e,f,g,h;
   isaac_ub4 *m,*r;
   ctx->randa = ctx->randb = ctx->randc = 0;
   m=ctx->randmem;
   r=ctx->randrsl;
   a=b=c=d=e=f=g=h=0x9e3779b9;  /* the golden ratio */

   for (i=0; i<4; ++i)          /* scramble it */
   {
     isaac_mix(a,b,c,d,e,f,g,h);
   }

   if (flag) 
   {
     /* initialize using the contents of r[] as the seed */
     for (i=0; i<ISAAC_RANDSIZ; i+=8)
     {
       a+=r[i  ]; b+=r[i+1]; c+=r[i+2]; d+=r[i+3];
       e+=r[i+4]; f+=r[i+5]; g+=r[i+6]; h+=r[i+7];
       isaac_mix(a,b,c,d,e,f,g,h);
       m[i  ]=a; m[i+1]=b; m[i+2]=c; m[i+3]=d;
       m[i+4]=e; m[i+5]=f; m[i+6]=g; m[i+7]=h;
     }
     /* do a second pass to make all of the seed affect all of m */
     for (i=0; i<ISAAC_RANDSIZ; i+=8)
     {
       a+=m[i  ]; b+=m[i+1]; c+=m[i+2]; d+=m[i+3];
       e+=m[i+4]; f+=m[i+5]; g+=m[i+6]; h+=m[i+7];
       isaac_mix(a,b,c,d,e,f,g,h);
       m[i  ]=a; m[i+1]=b; m[i+2]=c; m[i+3]=d;
       m[i+4]=e; m[i+5]=f; m[i+6]=g; m[i+7]=h;
     }
   }
   else
   {
     for (i=0; i<ISAAC_RANDSIZ; i+=8)
     {
       /* fill in mm[] with messy stuff */
       isaac_mix(a,b,c,d,e,f,g,h);
       m[i  ]=a; m[i+1]=b; m[i+2]=c; m[i+3]=d;
       m[i+4]=e; m[i+5]=f; m[i+6]=g; m[i+7]=h;
     }
   }

   isaac(ctx);            /* fill in the first set of results */
   ctx->randcnt=ISAAC_RANDSIZ;  /* prepare to use the first set of results */
}


#ifdef NEVER
int main()
{
  isaac_ub4 i,j;
  isaac_randctx ctx;
  ctx.randa=ctx.randb=ctx.randc=(isaac_ub4)0;
  for (i=0; i<256; ++i) ctx.randrsl[i]=(isaac_ub4)0;
  isaac_randinit(&ctx, ISAAC_TRUE);
  for (i=0; i<2; ++i)
  {
    isaac(&ctx);
    for (j=0; j<256; ++j)
    {
      printf("%.8lx",ctx.randrsl[j]);
      if ((j&7)==7) printf("\n");
    }
  }
}
#endif
